import { DUMMY_VIRTUAL_FILE_EXTENSIONS } from '../constants.js';
import * as Astro from './astro.js';
import * as MDX from './mdx.js';
import * as Svelte from './svelte.js';
import * as Vue from './vue.js';
const isAsync = (fn) => (fn ? fn.constructor.name === 'AsyncFunction' : false);
const normalizeExt = (ext) => ext.replace(/^\.*/, '.');
export const partitionCompilers = (rawLocalConfig) => {
    const syncCompilers = {};
    const asyncCompilers = {};
    for (const extension in rawLocalConfig.compilers) {
        const ext = normalizeExt(extension);
        if (!rawLocalConfig.asyncCompilers?.[ext] && isAsync(rawLocalConfig.compilers[extension])) {
            asyncCompilers[ext] = rawLocalConfig.compilers[extension];
        }
        else {
            syncCompilers[ext] = rawLocalConfig.compilers[extension];
        }
    }
    for (const extension in rawLocalConfig.asyncCompilers) {
        const ext = normalizeExt(extension);
        asyncCompilers[ext] = rawLocalConfig.asyncCompilers[extension];
    }
    return { ...rawLocalConfig, syncCompilers, asyncCompilers };
};
const compilers = new Map([
    ['.astro', Astro],
    ['.mdx', MDX],
    ['.svelte', Svelte],
    ['.vue', Vue],
]);
const dummyCompiler = () => '';
export const dummyCompilers = new Map(Array.from(DUMMY_VIRTUAL_FILE_EXTENSIONS).map(ext => [ext, dummyCompiler]));
const getDependencies = (manifest) => [
    ...Object.keys(manifest?.dependencies ?? {}),
    ...Object.keys(manifest?.devDependencies ?? {}),
];
export const mergeCompilers = (syncCompilers, asyncCompilers, manifest) => {
    const dependencies = getDependencies(manifest);
    const hasDependency = (packageName) => dependencies.includes(packageName);
    for (const [extension, { condition, compiler }] of compilers.entries()) {
        if (!syncCompilers.has(extension) && condition(hasDependency)) {
            syncCompilers.set(extension, compiler);
        }
    }
    return [syncCompilers, asyncCompilers];
};
export const getExtensions = (compilers) => [
    ...compilers[0].keys(),
    ...compilers[1].keys(),
];
